@@ -82,8 +82,9 @@ public:
 class KMPLAYER_NO_EXPORT Runtime {
 public:
     enum TimingState {
-        timings_reset = 0, timings_began,
-        timings_started, timings_paused, timings_stopped, timings_freezed
+        TimingsInit = 0, TimingsInitialized,
+        timings_began, timings_started, TimingsTransIn, timings_paused,
+        timings_stopped, timings_freezed
     };
     enum Fill {
         fill_default, fill_inherit, fill_remove, fill_freeze,
@@ -91,7 +92,7 @@ public:
     };
     enum DurationTime { BeginTime = 0, DurTime, EndTime, DurTimeLast };
     enum Duration {
-        DurInfinite = -1,
+        DurIndefinite = -1,
         DurTimer = (int) MsgEventTimer,
         DurActivated = (int) MsgEventClicked,
         DurInBounds = (int) MsgEventPointerInBounds,
@@ -115,19 +116,22 @@ public:
     /**
      * Reset all data, called from end() and init()
      */
-    void reset ();
+    void initialize ();
     bool parseParam (const TrieString & name, const String & value);
     TimingState state () const { return timingstate; }
-    void *message (MessageType msg, void *content=NULL);
-    void processEvent (MessageType msg);
+    void message (MessageType msg, void *content=NULL);
+    void *role (RoleType msg, void *content=NULL);
     /**
      * Duration items, begin/dur/end, length information or connected element
      */
     struct DurationItem {
-        DurationItem () : durval (DurTimer), offset (0) {}
+        DurationItem ();
+        DurationItem &operator = (const DurationItem &other);
+        void clear();
         Duration durval;
         int offset;
-        ConnectionPtr connection;
+        ConnectionLink connection;
+        DurationItem *next;
     } durations [(const int) DurTimeLast];
     void setDuration ();
     bool started () const {
@@ -144,9 +148,9 @@ public:
     TimingState timingstate;
     TimingState unpaused_state;
     int repeat_count;
-    NodeRefListPtr m_StartListeners;        // Element about to be started
-    NodeRefListPtr m_StartedListeners;      // Element is started
-    NodeRefListPtr m_StoppedListeners;      // Element stopped
+    ConnectionList m_StartListeners;        // Element about to be started
+    ConnectionList m_StartedListeners;      // Element is started
+    ConnectionList m_StoppedListeners;      // Element stopped
     Command *begin_timer;
     Command *duration_timer;
     Command *started_timer;
@@ -159,8 +163,8 @@ public:
     Fill fill_def;
     Fill fill_active;
     Element *element;
+    int trans_in_dur;
 private:
-    void setDurationItem (DurationTime item, const String & val);
     void propagateStop (bool forced);
     void propagateStart ();
     int repeat;
@@ -170,11 +174,11 @@ class KMPLAYER_NO_EXPORT MouseListeners {
 public:
     MouseListeners();
 
-    NodeRefList *receivers (MessageType msg);
+    ConnectionList *receivers (MessageType msg);
 
-    NodeRefListPtr m_ActionListeners;      // mouse clicked
-    NodeRefListPtr m_OutOfBoundsListeners; // mouse left
-    NodeRefListPtr m_InBoundsListeners;    // mouse entered
+    ConnectionList m_ActionListeners;      // mouse clicked
+    ConnectionList m_OutOfBoundsListeners; // mouse left
+    ConnectionList m_InBoundsListeners;    // mouse entered
 };
 
 /**
@@ -238,7 +242,7 @@ public:
     void deactivate ();
     void closed ();
     bool expose () const;
-    void *message (MessageType msg, void *content=NULL);
+    void message (MessageType msg, void *content=NULL);
     void accept (Visitor *v) { v->visit (this); }
     void jump (const String & id);
     static Smil * findSmilNode (Node * node);
@@ -256,7 +260,7 @@ public:
     Node *childFromTag (const String & tag);
     KDE_NO_EXPORT const char * nodeName () const { return "head"; }
     void closed ();
-    void *message (MessageType msg, void *content=NULL);
+    void message (MessageType msg, void *content=NULL);
     bool expose () const;
 };
 
@@ -269,7 +273,7 @@ public:
     Node *childFromTag (const String & tag);
     KDE_NO_EXPORT const char * nodeName () const { return "layout"; }
     void closed ();
-    void *message (MessageType msg, void *content=NULL);
+    void message (MessageType msg, void *content=NULL);
     void accept (Visitor *v) { v->visit (this); }
     bool expose () const { return false; }
 
@@ -288,7 +292,8 @@ public:
     virtual void activate ();
     virtual void deactivate ();
     virtual void parseParam (const TrieString & name, const String & value);
-    virtual void *message (MessageType msg, void *content=NULL);
+    virtual void message (MessageType msg, void *content=NULL);
+    virtual void *role (RoleType msg, void *content=NULL);
     virtual void accept (Visitor *v) { v->visit (this); }
     /**
      * repaints region, calls scheduleRepaint(x,y,w,h) on view
@@ -305,7 +310,7 @@ public:
     String background_image;
     ShowBackground show_background;
     Fit fit;
-    NodeRefListPtr m_AttachedMediaTypes;   // active attached mediatypes
+    ConnectionList m_AttachedMediaTypes;   // active attached mediatypes
     /**
      * boolean for check if pointerEntered/pointerLeft should be called by View
      */
@@ -326,7 +331,8 @@ public:
     ~RootLayout ();
     void closed ();
     void deactivate ();
-    void *message (MessageType msg, void *content=NULL);
+    void message (MessageType msg, void *content=NULL);
+    void *role (RoleType msg, void *content=NULL);
     KDE_NO_EXPORT const char * nodeName () const { return "root-layout"; }
 };
 
@@ -340,7 +346,8 @@ public:
     void deactivate ();
     KDE_NO_EXPORT const char * nodeName () const { return "region"; }
     Node *childFromTag (const String & tag);
-    void *message (MessageType msg, void *content=NULL);
+    void message (MessageType msg, void *content=NULL);
+    void *role (RoleType msg, void *content=NULL);
 private:
     MouseListeners mouse_listeners;
 };
@@ -423,13 +430,13 @@ public:
     void deactivate ();
     void reset ();
     bool expose () const { return false; }
-    void *message (MessageType msg, void *content=NULL);
+    void message (MessageType msg, void *content=NULL);
+    void *role (RoleType msg, void *content=NULL);
     void setJumpNode (NodePtr);
     Runtime *runtime;
 protected:
     GroupBase (NodePtr & d, short id);
     NodePtrW jump_node;
-    bool inited;
 };
 
 /**
@@ -441,7 +448,8 @@ public:
     KDE_NO_EXPORT const char * nodeName () const { return "par"; }
     void begin ();
     void reset ();
-    void *message (MessageType msg, void *content=NULL);
+    void message (MessageType msg, void *content=NULL);
+    void *role (RoleType msg, void *content=NULL);
     KDE_NO_EXPORT void accept (Visitor * v) { v->visit (this); }
 };
 
@@ -453,9 +461,11 @@ public:
     KDE_NO_CDTOR_EXPORT Seq (NodePtr & d) : GroupBase(d, id_node_seq) {}
     KDE_NO_EXPORT const char * nodeName () const { return "seq"; }
     void begin ();
-    void *message (MessageType msg, void *content=NULL);
+    void message (MessageType msg, void *content=NULL);
+    void *role (RoleType msg, void *content=NULL);
     KDE_NO_EXPORT void accept (Visitor * v) { v->visit (this); }
-    ConnectionPtr starting_connection;
+    ConnectionLink starting_connection;
+    ConnectionLink trans_connection;
 protected:
     KDE_NO_CDTOR_EXPORT Seq (NodePtr & d, short id) : GroupBase(d, id) {}
 };
@@ -475,17 +485,21 @@ public:
  */
 class KMPLAYER_NO_EXPORT Excl : public GroupBase {
 public:
-    KDE_NO_CDTOR_EXPORT Excl (NodePtr & d) : GroupBase (d, id_node_excl) {}
+    Excl (NodePtr & d);
+    ~Excl ();
     KDE_NO_EXPORT const char * nodeName () const { return "excl"; }
     Node *childFromTag (const String & tag);
     void begin ();
     void deactivate ();
-    void *message (MessageType msg, void *content=NULL);
+    void message (MessageType msg, void *content=NULL);
     KDE_NO_EXPORT void accept (Visitor * v) { v->visit (this); }
 
-    typedef ListNode <ConnectionPtr> ConnectionStoreItem;
-    List <ConnectionStoreItem> started_event_list;
-    ConnectionPtr stopped_connection;
+    struct ConnectionItem {
+        ConnectionItem (ConnectionItem *n) : next (n) {}
+        ConnectionLink link;
+        ConnectionItem *next;
+    } *started_event_list;
+    ConnectionLink stopped_connection;
     NodeRefList priority_queue;
     NodePtrW cur_node;
 };
@@ -501,7 +515,7 @@ public:
     Node *childFromTag (const String & tag);
     void init ();
     void parseParam (const TrieString &, const String &);
-    void *message (MessageType msg, void *content=NULL);
+    void message (MessageType msg, void *content=NULL);
     KDE_NO_EXPORT void accept (Visitor * v) { v->visit (this); }
 
     enum { PeersStop, PeersPause, PeersDefer, PeersNever } peers;
@@ -521,7 +535,7 @@ public:
     void begin ();
     void deactivate ();
     void reset ();
-    void *message (MessageType msg, void *content=NULL);
+    void message (MessageType msg, void *content=NULL);
     KDE_NO_EXPORT void accept (Visitor * v) { v->visit (this); }
 
     Node *chosenOne ();
@@ -535,8 +549,8 @@ public:
     void deactivate ();
     KDE_NO_EXPORT bool expose () const { return false; }
     void parseParam (const TrieString & name, const String & value);
-    ConnectionPtr mediatype_activated;
-    ConnectionPtr mediatype_attach;
+    ConnectionLink mediatype_activated;
+    ConnectionLink mediatype_attach;
     String href;
     String target;
     enum { show_new, show_replace } show;
@@ -549,7 +563,8 @@ public:
     Anchor (NodePtr & d);
     KDE_NO_CDTOR_EXPORT ~Anchor () {}
     void activate ();
-    void *message (MessageType msg, void *content=NULL);
+    void message (MessageType msg, void *content=NULL);
+    void *role (RoleType msg, void *content=NULL);
     KDE_NO_EXPORT const char * nodeName () const { return "a"; }
     Node *childFromTag (const String & tag);
     KDE_NO_EXPORT void accept (Visitor * v) { v->visit (this); }
@@ -563,7 +578,7 @@ public:
     KDE_NO_EXPORT const char * nodeName () const { return (const char *) tag; }
     KDE_NO_EXPORT void accept (Visitor * v) { v->visit (this); }
     void parseParam (const TrieString & name, const String & value);
-    void *message (MessageType msg, void *content=NULL);
+    void *role (RoleType msg, void *content=NULL);
     SizeType * coords;
     int nr_coords;
     const String tag;
@@ -591,7 +606,8 @@ public:
     virtual void reset ();
     SRect calculateBounds ();
     virtual void parseParam (const TrieString & name, const String & value);
-    virtual void *message (MessageType msg, void *content=NULL);
+    virtual void message (MessageType msg, void *content=NULL);
+    virtual void *role (RoleType msg, void *content=NULL);
     virtual void accept (Visitor *v) { v->visit (this); }
     bool expose () const { return false; }
 
@@ -623,15 +639,15 @@ protected:
     virtual void clipStop ();
 
     MouseListeners mouse_listeners;
-    NodeRefListPtr m_MediaAttached;        // mouse entered
-    ConnectionPtr region_paint;            // attached region needs painting
-    ConnectionPtr region_mouse_enter;      // attached region has mouse entered
-    ConnectionPtr region_mouse_leave;      // attached region has mouse left
-    ConnectionPtr region_mouse_click;      // attached region is clicked
-    ConnectionPtr region_attach;           // attached to region
-    ConnectionPtr document_postponed;      // pause audio/video accordantly
+    ConnectionList m_MediaAttached;        // mouse entered
+    ConnectionList m_TransformedIn;        // transIn ready
+    ConnectionLink region_paint;           // attached region needs painting
+    ConnectionLink region_mouse_enter;     // attached region has mouse entered
+    ConnectionLink region_mouse_leave;     // attached region has mouse left
+    ConnectionLink region_mouse_click;     // attached region is clicked
+    ConnectionLink region_attach;          // attached to region
+    ConnectionLink document_postponed;     // pause audio/video accordantly
     PostponePtr postpone_lock;
-    bool inited;
 };
 
 class KMPLAYER_NO_EXPORT RefMediaType : public MediaType {
@@ -654,7 +670,7 @@ public:
     virtual void activate ();
     virtual void accept (Visitor *);
     virtual void parseParam (const TrieString &, const String &);
-    virtual void *message (MessageType msg, void *content=NULL);
+    virtual void message (MessageType msg, void *content=NULL);
 };
 
 class KMPLAYER_NO_EXPORT TextMediaType : public MediaType {
@@ -695,7 +711,8 @@ public:
     KDE_NO_EXPORT const char *nodeName () const { return "smilText"; }
     Node *childFromTag (const String & tag);
     virtual void parseParam (const TrieString &name, const String &value);
-    virtual void *message (MessageType msg, void *content=NULL);
+    virtual void message (MessageType msg, void *content=NULL);
+    virtual void *role (RoleType msg, void *content=NULL);
     virtual void accept (Visitor *v) { v->visit (this); }
 
     String richText ();
@@ -703,10 +720,9 @@ public:
 
     SurfacePtrW text_surface;
     NodePtrW region_node;
-    ConnectionPtr region_attach;
+    ConnectionLink region_attach;
     MouseListeners mouse_listeners;
     Runtime *runtime;
-    bool inited;
 };
 
 class KMPLAYER_NO_EXPORT TextFlow : public Element {
@@ -751,7 +767,8 @@ public:
     virtual void reset ();
     virtual bool expose () const { return false; }
     virtual void parseParam (const TrieString & name, const String & value);
-    virtual void *message (MessageType msg, void *content=NULL);
+    virtual void message (MessageType msg, void *content=NULL);
+    virtual void *role (RoleType msg, void *content=NULL);
     Runtime *runtime;
 protected:
     void restoreModification ();
@@ -762,7 +779,6 @@ protected:
     String target_id;
     String change_to;
     int modification_id;
-    bool inited;
 };
 
 class KMPLAYER_NO_EXPORT Set : public AnimateGroup {
@@ -781,7 +797,7 @@ public:
     virtual void finish ();
     virtual void deactivate ();
     virtual void parseParam (const TrieString & name, const String & value);
-    virtual void *message (MessageType msg, void *content=NULL);
+    virtual void message (MessageType msg, void *content=NULL);
     virtual void accept (Visitor *v) { v->visit (this); }
     KDE_NO_EXPORT const char * nodeName () const { return "animate"; }
     PlayType playType () { return play_type_none; }
@@ -815,7 +831,7 @@ public:
     virtual void finish ();
     virtual void deactivate ();
     virtual void parseParam (const TrieString & name, const String & value);
-    virtual void *message (MessageType msg, void *content=NULL);
+    virtual void message (MessageType msg, void *content=NULL);
     virtual void accept (Visitor *v) { v->visit (this); }
     KDE_NO_EXPORT const char * nodeName () const { return "animateMotion"; }
     PlayType playType () { return play_type_none; }
