@@ -257,11 +257,11 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Smil *s) {
 
 KDE_NO_EXPORT void CairoPaintVisitor::traverseRegion (Node *node, Surface *s) {
     // next visit receivers
-    NodeRefListPtr nl = nodeMessageReceivers (node, MsgSurfaceAttach);
+    ConnectionList *nl = nodeMessageReceivers (node, MsgSurfaceAttach);
     if (nl) {
-        for (NodeRefItemPtr c = nl->first(); c; c = c->nextSibling ())
-            if (c->data)
-                c->data->accept (this);
+        for (Connection *c = nl->first(); c; c = nl->next ())
+            if (c->connecter)
+                c->connecter->accept (this);
     }
     /*for (SurfacePtr c = s->lastChild (); c; c = c->previousSibling ()) {
         if (c->node && c->node->id != SMIL::id_node_region &&
@@ -283,7 +283,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::traverseRegion (Node *node, Surface *s) {
 KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Layout *layout) {
     //debugLog() << "Visit " << layout->nodeName() << endl;
     if (layout->root_layout) {
-        Surface *s = (Surface *)layout->root_layout->message (MsgQueryRoleDisplay);
+        Surface *s = (Surface *)layout->root_layout->role (RoleDisplay);
         if (s) {
             //cairo_save (cr);
             Matrix m = matrix;
@@ -320,7 +320,7 @@ static void cairoDrawRect (cairo_t *cr, unsigned int color,
 }
 
 KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::RegionBase *reg) {
-    Surface *s = (Surface *) reg->message (MsgQueryRoleDisplay);
+    Surface *s = (Surface *) reg->role (RoleDisplay);
     if (s) {
         SRect rect = s->bounds;
 
@@ -343,7 +343,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::RegionBase *reg) {
              : NULL;
         ImageData *bg_img = im && !im->isEmpty() ? im->cached_img.ptr () : NULL;
         if ((SMIL::RegionBase::ShowAlways == reg->show_background ||
-                    reg->m_AttachedMediaTypes->first ()) &&
+                    reg->m_AttachedMediaTypes.first ()) &&
                 (s->background_color & 0xff000000 || bg_img)) {
             cairo_save (cr);
             if (s->background_color & 0xff000000) {
@@ -1192,7 +1192,7 @@ namespace KMPlayer {
 class KMPLAYER_NO_EXPORT MouseVisitor : public Visitor {
     ViewArea *view_area;
     Matrix matrix;
-    NodePtr node;
+    NodePtrW source;
     MessageType event;
     int x, y;
     bool handled;
@@ -1231,29 +1231,29 @@ KDE_NO_EXPORT void MouseVisitor::visit (SMIL::Smil *s) {
 }
 
 KDE_NO_EXPORT void MouseVisitor::visit (SMIL::Layout * layout) {
-    Surface *s = (Surface *) layout->root_layout->message (MsgQueryRoleDisplay);
+    Surface *s = (Surface *) layout->root_layout->role (RoleDisplay);
     if (s) {
         Matrix m = matrix;
         SRect rect = s->bounds;
         matrix = Matrix (rect.x(), rect.y(), s->xscale, s->yscale);
         matrix.transform (m);
 
-        NodePtr node_save = node;
-        node = layout;
+        NodePtr node_save = source;
+        source = layout;
         for (NodePtr r = layout->firstChild (); r; r = r->nextSibling ()) {
             if (r->id == SMIL::id_node_region)
                 r->accept (this);
-            if (!node->active ())
+            if (!source || !source->active ())
                 break;
         }
-        node = node_save;
+        source = node_save;
 
         matrix = m;
     }
 }
 
 KDE_NO_EXPORT void MouseVisitor::visit (SMIL::RegionBase *region) {
-    Surface *s = (Surface *) region->message (MsgQueryRoleDisplay);
+    Surface *s = (Surface *) region->role (RoleDisplay);
     if (s) {
         SRect rect = s->bounds;
         IRect scr = matrix.toScreen (rect);
@@ -1304,14 +1304,14 @@ KDE_NO_EXPORT void MouseVisitor::visit (SMIL::RegionBase *region) {
             for (NodePtr r = region->firstChild (); r; r = r->nextSibling ()) {
                 r->accept (this);
                 child_handled |= handled;
-                if (!node->active ())
+                if (!source || !source->active ())
                     break;
             }
         child_handled &= !bubble_up;
         bubble_up = false;
 
         MessageType saved_event = event;
-        if (node->active ()) {
+        if (source && source->active ()) {
             bool notify_receivers = !child_handled;
             bool pass_event = !child_handled;
             if (event == MsgEventPointerMoved) {
@@ -1331,12 +1331,12 @@ KDE_NO_EXPORT void MouseVisitor::visit (SMIL::RegionBase *region) {
                 region->deliver (event, &mouse_event);
             }
             if (pass_event) {
-                NodeRefList *nl = nodeMessageReceivers (region, MsgSurfaceAttach);
+                ConnectionList *nl = nodeMessageReceivers (region, MsgSurfaceAttach);
                 if (nl) {
-                    for (NodeRefItemPtr c = nl->first(); c; c = c->nextSibling ()) {
-                        if (c->data)
-                            c->data->accept (this);
-                        if (!node->active ())
+                    for (Connection *c = nl->first(); c; c = nl->next ()) {
+                        if (c->connecter)
+                            c->connecter->accept (this);
+                        if (!source || !source->active ())
                             break;
                     }
                 }
@@ -1384,7 +1384,7 @@ KDE_NO_EXPORT void MouseVisitor::visit (SMIL::Anchor * anchor) {
 
 KDE_NO_EXPORT void MouseVisitor::visit (SMIL::Area * area) {
     NodePtr n = area->parentNode ();
-    Surface *s = (Surface *) n->message (MsgQueryRoleDisplay);
+    Surface *s = (Surface *) n->role (RoleDisplay);
     if (s) {
         SRect rect = s->bounds;
         IRect scr = matrix.toScreen (rect);
@@ -1406,12 +1406,12 @@ KDE_NO_EXPORT void MouseVisitor::visit (SMIL::Area * area) {
         if (event == MsgEventPointerMoved)
             ;//cursor.setShape (Qt::PointingHandCursor);
         else {
-            NodeRefList *nl = nodeMessageReceivers (area, event);
+            ConnectionList *nl = nodeMessageReceivers (area, event);
             if (nl)
-                for (NodeRefItemPtr c = nl->first(); c; c = c->nextSibling ()) {
-                    if (c->data)
-                        c->data->accept (this);
-                    if (!node->active ())
+                for (Connection *c = nl->first(); c; c = nl->next ()) {
+                    if (c->connecter)
+                        c->connecter->accept (this);
+                    if (!source || !source->active ())
                         return;
                 }
             if (event == MsgEventClicked && !area->href.isEmpty ())
@@ -1421,9 +1421,11 @@ KDE_NO_EXPORT void MouseVisitor::visit (SMIL::Area * area) {
 }
 
 KDE_NO_EXPORT void MouseVisitor::visit (Element *elm) {
-    Runtime *rt = (Runtime *) elm->message (MsgQueryRoleTiming);
-    if (rt)
-        rt->processEvent (event);
+    Runtime *rt = (Runtime *) elm->role (RoleTiming);
+    if (rt) {
+        Posting mouse_event (source, elm, event);
+        rt->message (event, &mouse_event);
+    }
 }
 
 KDE_NO_EXPORT void MouseVisitor::visit (SMIL::MediaType *mt) {
@@ -1446,20 +1448,27 @@ KDE_NO_EXPORT void MouseVisitor::visit (SMIL::MediaType *mt) {
         return;
     mt->has_mouse = inside;
 
-    NodeRefList *nl = nodeMessageReceivers (mt,
+    ConnectionList *nl = nodeMessageReceivers (mt,
             event == MsgEventPointerMoved ? MsgSurfaceAttach : event);
-    if (nl)
-        for (NodeRefItemPtr c = nl->first(); c; c = c->nextSibling ()) {
-            if (c->data && c->data.ptr () != mt)
-                c->data->accept (this);
-            if (!node->active ())
-                return;
+    if (nl) {
+        NodePtr node_save = source;
+        source = mt;
+        for (Connection *c = nl->first(); c; c = nl->next ()) {
+            if (c->connecter && c->connecter.ptr () != mt)
+                c->connecter->accept (this);
+            if (!source || !source->active ())
+                break;
         }
+        source = node_save;
+        if (!source || !source->active ())
+            return;
+    }
+
     if (event != MsgEventPointerMoved)
         visit (static_cast <Element *> (mt));
     if (event != MsgEventPointerInBounds && event != MsgEventPointerOutBounds) {
       SMIL::RegionBase *r=convertNode<SMIL::RegionBase>(mt->region_node);
-      if (r && r->message (MsgQueryRoleDisplay) &&
+      if (r && r->role (RoleDisplay) &&
               r->id != SMIL::id_node_smil &&
               r->region_surface->node && r != r->region_surface->node.ptr ())
           return r->region_surface->node->accept (this);
@@ -1468,12 +1477,12 @@ KDE_NO_EXPORT void MouseVisitor::visit (SMIL::MediaType *mt) {
 
 KDE_NO_EXPORT void MouseVisitor::visit (SMIL::SmilText *st) {
     if (MsgEventClicked == event) {
-        NodeRefList *nl = nodeMessageReceivers (st, event);
+        ConnectionList *nl = nodeMessageReceivers (st, event);
         if (nl)
-            for (NodeRefItemPtr c = nl->first(); c; c = c->nextSibling ()) {
-                if (c->data && c->data.ptr () != st)
-                    c->data->accept (this);
-                if (!node->active ())
+            for (Connection *c = nl->first(); c; c = nl->next ()) {
+                if (c->connecter && c->connecter.ptr () != st)
+                    c->connecter->accept (this);
+                if (!source || !source->active ())
                     return;
             }
     }
